@@ -25,11 +25,11 @@ Params::Registry - Housekeeping for sets of named parameters
 
 =head1 VERSION
 
-Version 0.03
+Version 0.06
 
 =cut
 
-our $VERSION = '0.03';
+our $VERSION = '0.06';
 
 =head1 SYNOPSIS
 
@@ -371,7 +371,7 @@ sub process {
 
     my $instance = Params::Registry::Instance->new(registry => $self);
 
-    $instance->set($obj, -defaults => 1);
+    $instance->set($obj, -defaults => 1, -force => 1);
 }
 
 =head2 template $KEY
